//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Data/Data2DItem.h
//! @brief     Defines class Data2DItem.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_DATA_DATA2DITEM_H
#define BORNAGAIN_GUI_MODEL_DATA_DATA2DITEM_H

#include "GUI/Model/Data/DataItem.h"

class AmplitudeAxisItem;
class LineItem;
class MaskList;
class MaskeditorListmodel;
class ProjectionList;
class QCPColorGradient;

class Data2DItem : public DataItem {
    Q_OBJECT
public:
    static constexpr auto M_TYPE{"IntensityData"};

    Data2DItem();
    ~Data2DItem();

    void setDatafield(const Datafield& data) override;

    double xMin() const override;
    double xMax() const override;

    double yMin() const override;
    double yMax() const override;

    double zMin() const;
    double zMax() const;

    //... zoom ranges
    double lowerZ() const;
    double upperZ() const;
    void setZrange(double zmin, double zmax);
    void copyZRangeFromItem(DataItem* sourceItem);
    bool isZoomed() const;

    //! Color scheme of the color map
    QCPColorGradient currentGradientQCP() const;
    QString currentGradient() const;
    void setCurrentGradient(const QString& gradient);
    ComboProperty gradientCombo() const { return m_gradient; }

    //... logarithmic Z scale
    bool isLog() const;
    void setLog(bool islog);

    //... enable/disable colormap interpolation
    bool isInterpolated() const { return m_isInterpolated; }
    void setInterpolated(bool interp);

    //! True if min, max range of Z-axis is locked (change not allowed)
    bool isZaxisLocked() const;
    void setZaxisLocked(bool state);

    void updateDataRange();
    void computeDataRange();
    QPair<double, double> dataRange() const;

    const AmplitudeAxisItem* zAxisItem() const;
    AmplitudeAxisItem* zAxisItem();

    MaskList* maskContainerItem();
    const MaskList* maskContainerItem() const;
    MaskeditorListmodel* getOrCreateMaskModel();

    ProjectionList* projectionContainerItem();
    const ProjectionList* projectionContainerItem() const;
    MaskeditorListmodel* getOrCreateProjectionModel();

    void clearMasks();

    bool hasMasks() const;
    bool hasProjections() const;

    //! Returns dimensions and axesbins of data
    std::vector<int> axdims() const override;

    //! Set axes viewport to original data.
    void resetView() override;

    //! Sets zoom range of x,y axes to axes of input data
    void setAxesRangeToData();

    // write/read

    void writeTo(QXmlStreamWriter* w) const override;
    void readFrom(QXmlStreamReader* r) override;

signals:
    void gradientChanged();
    void interpolationChanged(bool isInterpol);
    void projectionCreated();
    void projectionPositionChanged(const LineItem* projection);
    void projectionGone(const LineItem* projection);
    void alignRanges();

private:
    void updateAxesZoomLevel();

    bool m_isInterpolated;
    ComboProperty m_gradient;
    std::unique_ptr<AmplitudeAxisItem> m_zAxis;
    std::unique_ptr<MaskeditorListmodel> m_mask_model;
    std::unique_ptr<MaskeditorListmodel> m_proj_model;
};

#endif // BORNAGAIN_GUI_MODEL_DATA_DATA2DITEM_H
