//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Model/DatafilesTree.cpp
//! @brief     Implements class DatafilesTree.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Model/DatafilesTree.h"
#include "GUI/Model/Device/DatafileItem.h"
#include "GUI/Model/Model/DatafilesModel.h"
#include <QApplication>
#include <QFontMetrics>
#include <QPalette>
#include <set>

namespace {

const std::set<size_t> availableRanks{1, 2};

} // namespace


DatafilesTree::DatafilesTree(QObject* parent, DatafilesModel* model)
    : QAbstractItemModel(parent)
    , m_model(model)
{
    for (auto rank : ::availableRanks)
        m_items[rank - 1] = model->realItems(rank);

    updateSubscriptions();
}

void DatafilesTree::refreshAfterModelChange()
{
    for (auto rank : ::availableRanks) {
        if (!m_items[rank - 1].isEmpty()) {
            beginRemoveRows(indexOfHeadline(rank), 0, m_items[rank - 1].size() - 1);
            m_items[rank - 1] = m_model->realItems(rank);
            endRemoveRows();
        }
    }

    updateSubscriptions();
}

void DatafilesTree::removeDataItem(DatafileItem* item)
{
    QModelIndex index = indexForItem(item);
    if (!index.isValid())
        return;

    const int rank = item->rank();
    const int rowOfItem = m_items[rank - 1].indexOf(item);
    beginRemoveRows(indexOfHeadline(rank), rowOfItem, rowOfItem);
    m_items[rank - 1].removeAll(item);
    m_model->removeDatafileItem(item);
    endRemoveRows();
}

DatafileItem* DatafilesTree::injectDataItem(int rank)
{
    auto* newItem = m_model->insertDataItem(rank);
    const int rowOfItem = m_model->realItems(rank).indexOf(newItem);
    beginInsertRows(indexOfHeadline(rank), rowOfItem, rowOfItem);
    m_items[rank - 1] = m_model->realItems(rank);
    endInsertRows();
    updateSubscriptions();
    return newItem;
}

DatafileItem* DatafilesTree::topMostItem() const
{
    if (!m_items[0].isEmpty())
        return m_items[0].first();
    if (!m_items[1].isEmpty())
        return m_items[1].first();
    return nullptr;
}

QModelIndex DatafilesTree::indexOfHeadline(int rank) const
{
    return createIndex(rank - 1, 0, nullptr);
}

QModelIndex DatafilesTree::index(int row, int column, const QModelIndex& parent) const
{
    if (!hasIndex(row, column, parent))
        return {};

    if (!parent.isValid())
        return createIndex(row, column, nullptr);

    for (auto rank : ::availableRanks)
        if (parent == indexOfHeadline(rank))
            return createIndex(row, column, m_items[rank - 1][row]);

    return {};
}

QModelIndex DatafilesTree::parent(const QModelIndex& index) const
{
    if (!index.isValid())
        return {};

    if (index.internalPointer() == nullptr) // index is headline => no parent
        return {};

    return indexOfHeadline(itemForIndex(index)->rank());
}

int DatafilesTree::columnCount(const QModelIndex& /*parent*/) const
{
    return 1;
}

int DatafilesTree::rowCount(const QModelIndex& parent) const
{
    if (!parent.isValid())
        return ::availableRanks.size();

    // parent is a headline
    for (int rank : ::availableRanks)
        if (parent == indexOfHeadline(rank))
            return m_items[rank - 1].size();

    return 0;
}

QVariant DatafilesTree::data(const QModelIndex& index, int role) const
{
    if (isHeadline(index)) {
        QString title = (index == indexOfHeadline(1)) ? "1D Data" : "2D Data";

        switch (role) {
        case Qt::DisplayRole:
            return title;

        case Qt::FontRole: {
            QFont f(QApplication::font());
            f.setPointSize(f.pointSize() * 1.5);
            f.setBold(true);
            return f;
        }

        case Qt::SizeHintRole: {
            QFont f(QApplication::font());
            f.setPointSize(f.pointSize() * 1.5);
            f.setBold(true);
            QSize s = QFontMetrics(f).boundingRect(title).size();
            return QSize(s.width() * 2, s.height() * 2);
        }

        case Qt::TextAlignmentRole:
            return QVariant(Qt::AlignLeft | Qt::AlignVCenter);

        case Qt::BackgroundRole:
            return qApp->palette().color(QPalette::Base);

        case Qt::ForegroundRole:
            return qApp->palette().color(QPalette::Text);

        default:
            return {};
        }
    }

    auto* const item = itemForIndex(index);

    if (role == Qt::ToolTipRole)
        return {};

    if (role == Qt::DisplayRole)
        return item->realItemName();

    if (role == Qt::EditRole)
        return item->realItemName();

    return {};
}

Qt::ItemFlags DatafilesTree::flags(const QModelIndex& index) const
{
    if (isHeadline(index) || !index.isValid())
        return Qt::NoItemFlags;

    auto f = QAbstractItemModel::flags(index);
    f |= Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled;

    if (index.column() == 0) // col 0 contains name of the data entry
        f |= Qt::ItemIsEditable;

    return f;
}

bool DatafilesTree::setData(const QModelIndex& index, const QVariant& value, int role)
{
    if (!index.isValid())
        return false;

    if (role == Qt::EditRole && index.column() == 0) {
        itemForIndex(index)->setDatafileItemName(value.toString());
        emit dataChanged(index, index);
        return true;
    }

    return false;
}

DatafileItem* DatafilesTree::itemForIndex(const QModelIndex& index) const
{
    if (!index.isValid())
        return nullptr;

    return reinterpret_cast<DatafileItem*>(index.internalPointer());
}

QModelIndex DatafilesTree::indexForItem(DatafileItem* item) const
{
    if (item == nullptr)
        return {};

    const int rank = item->rank();
    if (auto index = m_items[rank - 1].indexOf(item); index >= 0)
        return createIndex(index, 0, item);

    return {};
}

bool DatafilesTree::isHeadline(const QModelIndex& index) const
{
    if (!index.isValid())
        return false;

    return index.internalPointer() == nullptr;
}

void DatafilesTree::updateSubscriptions()
{
    for (auto* item : m_items[0])
        connect(item, &DatafileItem::importContentsProcessed, this,
                [this, item] { onContentsProcessed(item); });

    for (auto* item : m_items[1])
        connect(item, &DatafileItem::importContentsProcessed, this,
                [this, item] { onContentsProcessed(item); });
}

void DatafilesTree::onContentsProcessed(DatafileItem* item)
{
    QModelIndex index = indexForItem(item);
    emit dataChanged(index, index);
}
