//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/ScanEditor.cpp
//! @brief     Defines class ScanEditor.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/ScanEditor.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Beam/BeamWavelengthItem.h"
#include "GUI/Model/Beam/SourceItems.h"
#include "GUI/Model/Device/InstrumentNotifier.h"
#include "GUI/View/Device/FootprintForm.h"
#include "GUI/View/Instrument/AlphaScanEditor.h"
#include "GUI/View/Instrument/DistributionEditor.h"
#include "GUI/View/Numeric/DoubleSpinBox.h"
#include <QFormLayout>
#include <QLineEdit>

ScanEditor::ScanEditor(QWidget* parent, ScanItem* item, InstrumentNotifier* ec,
                       bool allow_footprint, bool allow_distr)
    : CollapsibleGroupBox("Beam and scan parameters", parent, item->expandBeamParameters)
{
    auto* layout = new QVBoxLayout;
    body()->setLayout(layout);

    //... Intensity

    auto* form = new QFormLayout;
    layout->addLayout(form);
    form->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
    auto* intensity_spinbox = new DoubleSpinBox(&item->intensity());
    form->addRow("Intensity:", intensity_spinbox);
    intensity_spinbox->setToolTip("Correction factor for normalized intensity");

    //... Wavelength and angles of incidence

    auto* hLayout = new QHBoxLayout;
    hLayout->setAlignment(Qt::AlignLeft);
    layout->addLayout(hLayout);

    auto* wavelengthEditor = new DistributionEditor("Wavelength (nm)", MeanConfig{true},
                                                    GUI::ID::Distributions::Symmetric, this,
                                                    item->wavelengthItem(), allow_distr);
    hLayout->addWidget(wavelengthEditor);

    auto* inclinationEditor = new AlphaScanEditor(this, item->grazingScanItem(), allow_distr);
    hLayout->addWidget(inclinationEditor);

    // order of the next two connections is important! Indicators have to be recalculated first,
    // then updated (recalculation is done in EditController)
    connect(wavelengthEditor, &DistributionEditor::distributionChanged, ec,
            &InstrumentNotifier::notifyWavelengthDistributionChanged);
    connect(wavelengthEditor, &DistributionEditor::distributionChanged, inclinationEditor,
            &AlphaScanEditor::updateIndicators);

    // order of the next two connections is important! Wavelength has to be recalculated first,
    // then updated (recalculation is done in EditController)
    connect(inclinationEditor, &AlphaScanEditor::dataChanged, ec,
            &InstrumentNotifier::notifyInclinationAxisChanged);
    connect(inclinationEditor, &AlphaScanEditor::dataChanged, wavelengthEditor,
            &DistributionEditor::updateData);

    //... Footprint

    if (allow_footprint) {
        auto* footprintEditor = new FootprintForm(this, item);
        hLayout->addWidget(footprintEditor);
        connect(footprintEditor, &FootprintForm::dataChanged, this, &ScanEditor::dataChanged);
    }
}
