//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Job/JobResultsPresenter.cpp
//! @brief     Implements class JobResultsPresenter.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Job/JobResultsPresenter.h"
#include "GUI/Model/Job/JobItem.h"
#include "GUI/View/Frame/Fit1DFrame.h"
#include "GUI/View/Frame/Fit2DFrame.h"
#include "GUI/View/Frame/Plot2DFrame.h"
#include "GUI/View/Frame/Plot2DWithProjectionFrame.h"
#include "GUI/View/Frame/SpecularFrame.h"
#include "GUI/View/Job/JobViewActivities.h"

namespace {

QString fitPresentationOfInstrument(JobItem* ji)
{
    return ji->rank() == 1 ? "Fit 1D Data" : "Fit 2D Data";
}

QStringList defaultActivePresentationsOfInstrument(JobItem* ji)
{
    if (ji->rank() == 1)
        return {"Reflectometry"};
    return {"Heat Map", "Projections"};
}

QString defaultPresentationOfInstrument(JobItem* ji)
{
    return ji->rank() == 1 ? "Reflectometry" : "Heat Map";
}

} // namespace


JobResultsPresenter::JobResultsPresenter()
{
    setWindowTitle("Job Datafield");
    setObjectName("JobResultsPresenter");

    registerWidget("Heat Map", [] { return new Plot2DFrame; });
    registerWidget("Projections", [] { return new Plot2DWithProjectionFrame; });
    registerWidget("Fit 1D Data", [] { return new Fit1DFrame; });
    registerWidget("Fit 2D Data", [] { return new Fit2DFrame; });
    registerWidget("Reflectometry", [] { return new SpecularFrame; });

    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    setMinimumSize(220, 200);
}

QString JobResultsPresenter::itemPresentation() const
{
    if (!jobItem())
        return selectedPresentation();

    if (jobItem()->presentationType().isEmpty())
        return defaultPresentation();

    return jobItem()->presentationType();
}

void JobResultsPresenter::setJobItem(JobItem* jobItem)
{
    setTheItem(jobItem);
}

void JobResultsPresenter::setThePresentation(const QString& type)
{
    setComboPresentation(type);
    if (jobItem() && !type.isEmpty())
        jobItem()->setPresentationType(type);
}

void JobResultsPresenter::setPresentationForActivity(JobViewActivity activity)
{
    if (!jobItem() || jobItem()->isRunning())
        return;

    QString presentation = "";
    switch (activity) {
    case JobViewActivity::Fitting:
        presentation = fitPresentationOfInstrument(jobItem());
        break;

    case JobViewActivity::RealTime:
    case JobViewActivity::JobView:
        presentation = defaultPresentationOfInstrument(jobItem());
        break;
    }
    setThePresentation(presentation);
}

//! Returns list of presentation types, available for given item. JobItem with fitting abilities
//! is valid for Plot2DFrame and Fit2DFrame.

QStringList JobResultsPresenter::activePresentationList() const
{
    if (!jobItem() || jobItem()->isRunning())
        return {};

    auto result = defaultActivePresentationsOfInstrument(jobItem());

    if (jobItem()->isValidForFitting())
        result << fitPresentationOfInstrument(jobItem());

    return result;
}

QStringList JobResultsPresenter::presentationList() const
{
    if (!jobItem() || jobItem()->isRunning())
        return {};

    auto result = defaultActivePresentationsOfInstrument(jobItem());
    result << fitPresentationOfInstrument(jobItem());

    return result;
}
